#define VC_EXTRALEAN
#include <windows.h>
#include <locale.h>
#define _FAR_NO_NAMELESS_UNIONS
#include "plugin.hpp"
#include "translng.hpp"
#include "translit.hpp"

#include "trans.cpp"
#include "filemix.cpp"
#include "ProcessName.cpp"

static char szModulePath[_MAX_PATH];

BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID )
{
    if( fdwReason==DLL_PROCESS_ATTACH ){
        GetModuleFileName(hinstDLL,szModulePath,sizeof(szModulePath));
        strrchr(szModulePath,'\\')[1]=0;
    }
    return TRUE;
}

void WINAPI _export SetStartupInfo(const struct PluginStartupInfo *Info)
{
    ::Info=*Info;
    IsOldFar=TRUE;
    if(Info->StructSize >= sizeof(struct PluginStartupInfo))
    {
      ::FSF=*Info->FSF;
      ::Info.FSF=&::FSF;
      IsOldFar=FALSE;
    }
}


HANDLE WINAPI _export OpenPlugin(int /*OpenFrom*/,int /*Item*/)
{
    if(!IsOldFar){

        int no;
        struct map_list_t *map_list=LoadTables(szModulePath);

        if( map_list==0 ){
            const char* items[]={
                Info.GetMsg(Info.ModuleNumber,MTranslit),
                Info.GetMsg(Info.ModuleNumber,MNoTables)
            };
            Info.Message(Info.ModuleNumber,FMSG_WARNING|FMSG_MB_OK,"Contents",items,sizeof(items)/sizeof(items[0]),0);
            return INVALID_HANDLE_VALUE;
        }

        {
            for( no=0; map_list[no].map; no++ ) continue;
            struct FarMenuItem *items=(struct FarMenuItem*)calloc(no,sizeof(struct FarMenuItem));
            for( no=0; map_list[no].map; no++ ) {
                strncpy(items[no].Text,map_list[no].name,sizeof(items[no].Text));
            }

            items[0].Selected=TRUE;

            no=Info.Menu(Info.ModuleNumber,-1,-1,0,FMENU_WRAPMODE|FMENU_AUTOHIGHLIGHT,Info.GetMsg(Info.ModuleNumber,MSelTable),0,"Contents",0,0,items,no);
            free(items);
            if( no==-1 ){
                FreeTables(map_list);
                return INVALID_HANDLE_VALUE;
            }
        }

        IsOldWin=TRUE;
        OSVERSIONINFO ovi={sizeof(OSVERSIONINFO)};
        if( GetVersionEx(&ovi) )
            if( ovi.dwPlatformId==VER_PLATFORM_WIN32_NT )
                IsOldWin=FALSE;

        if( setlocale(LC_ALL,IsOldWin?".ACP":map_list[no].cp)==0 ){
            const char *items[]={
                Info.GetMsg(Info.ModuleNumber,MTranslit),
                Info.GetMsg(Info.ModuleNumber,MErrSetlocale),
                map_list[no].cp
            };
            Info.Message(Info.ModuleNumber,FMSG_WARNING|FMSG_MB_OK,"tbl",items,sizeof(items)/sizeof(items[0]),0);
            FreeTables(map_list);
            return INVALID_HANDLE_VALUE;
        }


        struct PanelInfo PInfo;
        Info.Control(INVALID_HANDLE_VALUE,FCTL_GETPANELINFO,&PInfo);

        HANDLE hScreen=Info.SaveScreen(0,0,-1,-1);
        const char *MsgItems[]={Info.GetMsg(Info.ModuleNumber,MTranslit),Info.GetMsg(Info.ModuleNumber,MConverting)};
        Info.Message(Info.ModuleNumber,0,"Contents",MsgItems,sizeof(MsgItems)/sizeof(MsgItems[0]),0);

        char FullName[NM];

        for (int I=0;I < PInfo.SelectedItemsNumber; I++)
        {
          GetFullName(FullName,PInfo.CurDir,
                      PInfo.SelectedItems[I].FindData.cFileName);

          ProcessName(FullName,PInfo.SelectedItems[I].FindData.dwFileAttributes,map_list[no].map);
        }

        Info.RestoreScreen(hScreen);
        Info.Control(INVALID_HANDLE_VALUE,FCTL_UPDATEPANEL,NULL);
        Info.Control(INVALID_HANDLE_VALUE,FCTL_REDRAWPANEL,NULL);

        FreeTables(map_list);
    }
    return(INVALID_HANDLE_VALUE);
}


void WINAPI _export GetPluginInfo(struct PluginInfo *Info)
{
    if(!IsOldFar)
    {
      Info->StructSize=sizeof(*Info);
      static char *PluginMenuStrings[1];
      PluginMenuStrings[0]=(char*)::Info.GetMsg(::Info.ModuleNumber,MTranslit);
      Info->PluginMenuStrings=PluginMenuStrings;
      Info->PluginMenuStringsNumber=sizeof(PluginMenuStrings)/sizeof(PluginMenuStrings[0]);
    }
}
