#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <share.h>
#include <malloc.h>

struct translit_map_t {
    char* from;
    char* to;
};

struct map_list_t{
    struct translit_map_t* map;
    char*         name;
    char*         cp;
};

char* decode(char *buff)
{
    char* p=buff,*e;
    while( (p=strchr(p,'\\'))!=0 ){
        if( p[1]=='x' || p[1]=='X' ){
            *p++=(char)(strtol(&p[2],&e,16)&0xff);
            memmove(p,e,strlen(e)+1);
        }else if( isdigit((unsigned char)p[1]) ){
            *p++=(char)(strtol(&p[2],&e,10)&0xff);
            memmove(p,e,strlen(e)+1);
        }else p++;
    }
    return buff;
}

struct translit_map_t* LoadTable(const char* path, struct map_list_t* item)
{
    item->cp=0;
    item->name=0;
    item->map=0;

    FILE *f=_fsopen(path,"rt",_SH_DENYWR);
    if( !f ){
        const char *items[]={
            Info.GetMsg(Info.ModuleNumber,MTranslit),
            Info.GetMsg(Info.ModuleNumber,MCannotOpen),
            path
        };
        Info.Message(Info.ModuleNumber,FMSG_WARNING|FMSG_MB_OK|FMSG_ERRORTYPE,"Contents",items,sizeof(items)/sizeof(items[0]),0);
        return 0;
    }
    char buff[_MAX_PATH];
    int cnt=0;
    static const char delim[]=" \t\n\r";
    static const char delim2[]="\n\r";

    while( fgets(buff,sizeof(buff),f) ){
        char *p;

        p=strtok(buff,delim);
        if( p==0 || *p=='/' || *p=='*' || *p==':' || *p==0 ) continue;
        if( strnicmp(p,"?chcp",5)==0 ){
            p=strtok(0,delim2);
            if( p ){
                if( item->cp ) free(item->cp);
                item->cp=strdup(p);
            }
            continue;
        }else if( strnicmp(p,"?name",5)==0 ){
            p=strtok(0,delim2);
            if( p ){
                if( item->name ) free(item->name);
                item->name=strdup(p);
            }
            continue;
        }
        if( *p=='?' ) continue;
        item->map=(struct translit_map_t*)realloc(item->map,sizeof(struct translit_map_t)*(++cnt+1));
        item->map[cnt-1].from=strdup(decode(p));
        p=strtok(0,delim);
        item->map[cnt-1].to=strdup(p?decode(p):"");
    }

    fclose(f);

    if( cnt==0 ){
        free(item->cp);
        free(item->name);
        return 0;
    }
    item->map[cnt].from=0;
    item->map[cnt].to=0;
    if( !item->cp ) item->cp=strdup(".OCP");
    if( !item->name ) item->name=strdup(GetOnlyName((char*)path));
    return item->map;
}

void FreeTable(struct translit_map_t* map)
{
    for( int i=0; map[i].from!=0; i++ ){
        free(map[i].from);
        free(map[i].to);
    }
    free(map);
}

void FreeTables(struct map_list_t* list)
{
    for( int i=0; list[i].map; i++ ){
        FreeTable(list[i].map);
        free(list[i].name);
        free(list[i].cp);
    }
    free(list);
}

int __cdecl cmplist(const void *elem1, const void *elem2 )
{
    return strcmp(((struct map_list_t*)elem1)->name,((struct map_list_t*)elem2)->name);
}

struct map_list_t* LoadTables(const char* path)
{
    HANDLE hFind;
    char pn[_MAX_PATH];
    WIN32_FIND_DATA fd;
    struct map_list_t* list=0;
    size_t cnt=0;

    sprintf(pn,"%s*.tbl",path);
    hFind=FindFirstFile(pn,&fd);
    if( hFind==INVALID_HANDLE_VALUE ) return 0;
    do{
        if( !(fd.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) ){
            list=(struct map_list_t*)realloc(list,sizeof(struct map_list_t)*(++cnt+1));
            if(!LoadTable(GetFullName(pn,path,fd.cFileName),&list[cnt-1])) cnt--;
        }
    }while(FindNextFile(hFind,&fd));
    FindClose(hFind);
    list[cnt].map=0;
    list[cnt].name=0;
    list[cnt].cp=0;
    if( cnt==0 ){
        FreeTables(list);
        return 0;
    }
    qsort(list,cnt,sizeof(struct map_list_t),cmplist);
    return list;
}

void Translit(char* buf, const char* in, const struct translit_map_t *map)
{
    char* out=buf;
    int dout=0;
    int i;

    if( IsOldWin ) { //W9X
        char *p = (char*) alloca(strlen(in)+1);
        if( !p ) {
            strcpy(out,in);
            return;
        }
        OemToChar(in,p);
        in = p;
        for (i=0; map[i].from!=0; i++) {
            OemToChar(map[i].from,map[i].from);
            OemToChar(map[i].to,map[i].to);
        }
    }

    while (*in) {
        for (i=0; map[i].from!=0; i++){
            if(!strnicmp(in,map[i].from,strlen(map[i].from))){
                 dout=sprintf(out,"%s",map[i].to);
                 break;
            }
        }
        if (map[i].from==0) {*out++=*in++;continue;}
        if (isupper((unsigned char)(*in))) *out=(char)toupper(*out);
        in++;
        out+=dout;
    }

    *out=0;

    if( IsOldWin ) CharToOem(out,out);
}
